@@ -1,6 +1,6 @@
 # -*- perl -*-
 #
-# Copyright (C) 2011-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2011-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -158,6 +158,8 @@ our %EXPORT_TAGS = ( 'zone' => [ qw/ZONE_FRAME ZONE_NAME ZONE_DIGIT ZONE_BOX/ ],
 		     'position'=> [ qw/POSITION_BOX POSITION_MEASURE/ ],
 		   );
 
+use DBI qw(:sql_types);
+
 use AMC::Basic;
 use AMC::DataModule;
 use XML::Simple;
@@ -167,28 +169,66 @@ use XML::Simple;
 use_gettext();
 
 sub version_current {
-  return(1);
+  return(3);
 }
 
 sub version_upgrade {
     my ($self,$old_version)=@_;
     if($old_version==0) {
 
-	# Upgrading from version 0 (empty database) to version 1 :
-	# creates all the tables.
+      # Upgrading from version 0 (empty database) to version 1 :
+      # creates all the tables.
 
-	debug "Creating capture tables...";
-	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("page")
-		      ." (src TEXT, student INTEGER, page INTEGER, copy INTEGER DEFAULT 0, timestamp_auto INTEGER DEFAULT 0, timestamp_manual INTEGER DEFAULT 0, a REAL, b REAL, c REAL, d REAL, e REAL, f REAL, mse REAL, layout_image TEXT, annotated TEXT, timestamp_annotate INTEGER, PRIMARY KEY (student,page,copy))");
-	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("zone")
-		      ." (zoneid INTEGER PRIMARY KEY, student INTEGER, page INTEGER, copy INTEGER, type INTEGER, id_a INTEGER, id_b INTEGER, total INTEGER DEFAULT -1, black INTEGER DEFAULT -1, manual REAL DEFAULT -1, image TEXT)");
-	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("position")
-		      ." (zoneid INTEGER, corner INTEGER, x REAL, y REAL, type INTEGER, PRIMARY KEY (zoneid,corner,type))");
-	$self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("failed")
-		      ." (filename TEXT UNIQUE, timestamp INTEGER)");
-	$self->populate_from_xml;
+      debug "Creating capture tables...";
+      $self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("page")
+		    ." (src TEXT, student INTEGER, page INTEGER, copy INTEGER DEFAULT 0, timestamp_auto INTEGER DEFAULT 0, timestamp_manual INTEGER DEFAULT 0, a REAL, b REAL, c REAL, d REAL, e REAL, f REAL, mse REAL, layout_image TEXT, annotated TEXT, timestamp_annotate INTEGER, PRIMARY KEY (student,page,copy))");
+      $self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("zone")
+		    ." (zoneid INTEGER PRIMARY KEY, student INTEGER, page INTEGER, copy INTEGER, type INTEGER, id_a INTEGER, id_b INTEGER, total INTEGER DEFAULT -1, black INTEGER DEFAULT -1, manual REAL DEFAULT -1, image TEXT)");
+      $self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("position")
+		    ." (zoneid INTEGER, corner INTEGER, x REAL, y REAL, type INTEGER, PRIMARY KEY (zoneid,corner,type))");
+      $self->sql_do("CREATE TABLE IF NOT EXISTS ".$self->table("failed")
+		    ." (filename TEXT UNIQUE, timestamp INTEGER)");
+      $self->populate_from_xml;
 
-	return(1);
+      return(1);
+    } elsif($old_version==1) {
+
+      # Includes zoom files in the database
+
+      debug "Including zoom files in the database...";
+      $self->sql_do("ALTER TABLE ".$self->table("zone")
+		    ." ADD COLUMN imagedata BLOB");
+
+      my $list=$self->get_image_paths(ZONE_BOX);
+      my $nn=1+$#{$list};
+
+      if($nn>0) {
+	$self->progression('begin',__"Including zooms in the database...");
+
+	my $ii=0;
+	for my $i (@{$list}) {
+	  if($i->{image}) {
+	    my $f=$self->{'data'}->{'directory'}."/../cr/zooms/".$i->{image};
+	    if(defined($f) && -f $f) {
+	      $self->set_image($i->{zoneid},file_content($f));
+	    }
+	  }
+	  $ii++;
+	  $self->progression('fraction',$ii/$nn);
+	}
+
+	$self->progression('end');
+      }
+
+      return(2)
+    } elsif($old_version==2) {
+      $self->progression('begin',__("Building capture database indexes..."));
+      $self->sql_do("CREATE UNIQUE INDEX IF NOT EXISTS "
+		    .$self->index("index_zone")." ON "
+		    .$self->table("zone","self")." (student,page,copy,type,id_a,id_b)");
+
+      $self->progression('end');
+      return(3);
     }
     return('');
 }
@@ -280,8 +320,7 @@ sub populate_from_xml {
 	  # Look if the namefield image is present...
 	  my $nf="nom-".id2idf($id).".jpg";
 	  if(-f "$cr/$nf") {
-	    $self->statement('setZoneAuto')
-	      ->execute(-1,-1,$nf,$zoneid);
+	    $self->set_zone_auto_id_without_imagedata($zoneid,-1,-1,$nf);
 	  }
 	}
 
@@ -295,9 +334,10 @@ sub populate_from_xml {
 			 $case->{'question'},$case->{'reponse'});
 	  $zf='' if(!-f "$cr/zooms/$zf");
 
-	  $self->statement('setZoneAuto')
-	    ->execute($case->{'pixels'},$case->{'pixelsnoirs'},$zf,
-		      $zoneid);
+	  $self->set_zone_auto_id_without_imagedata
+	    ($zoneid,
+	     $case->{'pixels'},$case->{'pixelsnoirs'},
+	     $zf);
 
 	  $self->populate_position($case->{'coin'},$zoneid,POSITION_BOX);
 	  $self->populate_position($an->{'casetest'}->{$c}->{'coin'},
@@ -371,7 +411,9 @@ sub define_statements {
      'setZoneManual'=>{'sql'=>"UPDATE $t_zone"
 		       ." SET manual=? WHERE zoneid=?"},
      'setZoneAuto'=>{'sql'=>"UPDATE $t_zone"
-		     ." SET total=?, black=?, image=? WHERE zoneid=?"},
+		     ." SET total=?, black=?, image=?, imagedata=? WHERE zoneid=?"},
+     'setZoneAutoPrim'=>{'sql'=>"UPDATE $t_zone"
+			 ." SET total=?, black=?, image=? WHERE zoneid=?"},
      'nPages'=>{'sql'=>"SELECT COUNT(*) FROM $t_page"
 		." WHERE timestamp_auto>0 OR timestamp_manual>0"},
      'nPagesAuto'=>{'sql'=>"SELECT COUNT(*) FROM $t_page"
@@ -430,13 +472,24 @@ sub define_statements {
      'pageNearRatio'=>{'sql'=>"SELECT MIN(ABS(1.0*black/total-?))"
 		       ." FROM $t_zone"
 		       ." WHERE student=? AND page=? AND copy=? AND total>0"},
-     'pageZones'=>{'sql'=>"SELECT * FROM $t_zone"
+     'pageZones'=>{'sql'=>"SELECT zoneid,id_a,id_b,total,black,manual FROM $t_zone"
 		   ." WHERE student=? AND page=? AND copy=? AND type=?"},
      'zonesImages'=>{'sql'=>"SELECT image FROM $t_zone"
 		     ." WHERE student=? AND page=? AND copy=? AND type=?"},
+     'imagePaths'=>{'sql'=>"SELECT zoneid,image FROM $t_zone WHERE type=?"},
+     'setImage'=>{'sql'=>"UPDATE $t_zone SET image='',imagedata=? WHERE zoneid=?"},
+     'zoomsTotalSize'=>{'sql'=>"SELECT SUM(LENGTH(imagedata)) FROM $t_zone"
+			." WHERE type=?"},
+     'zoomsCleanup'=>{'sql'=>"UPDATE $t_zone SET imagedata=NULL WHERE type=?"},
      'pageZonesAll'=>{'sql'=>"SELECT * FROM $t_zone"
 		      ." WHERE type=?"},
-     'pageZonesD'=>{'sql'=>"SELECT * FROM $t_zone"
+     'pageZonesD'=>{'sql'=>"SELECT zoneid,id_a,id_b,total,black,manual"
+		    ." FROM $t_zone"
+		    ." WHERE student=? AND page=? AND copy=? AND type=?"
+		    ." AND total>0"
+		    ." ORDER BY 1.0*black/total"},
+     'pageZonesDI'=>{'sql'=>"SELECT *"
+		    ." FROM $t_zone"
 		    ." WHERE student=? AND page=? AND copy=? AND type=?"
 		    ." AND total>0"
 		    ." ORDER BY 1.0*black/total"},
@@ -509,7 +562,7 @@ sub define_statements {
 		    ." WHERE zoneid=? AND type=?"},
      'zoneDist'=>{'sql'=>"SELECT AVG((x-?)*(x-?)+(y-?)*(y-?))"
 		  ." FROM $t_position"
-		  ." WHERE zoneid=? AND TYPE=?"},
+		  ." WHERE zoneid=? AND type=?"},
      'getAnnotated'=>{'sql'=>"SELECT annotated,timestamp_annotate,student,page,copy"
 		      ." FROM $t_page"
 		      ." WHERE timestamp_annotate>0"
@@ -575,14 +628,30 @@ sub define_statements {
       ."      AS c, $t_lnf AS l"
       ."     ON c.student=l.student AND c.page=l.page ) AS a"
       ." LEFT OUTER JOIN"
-      ." ( SELECT * FROM $t_zone WHERE type=? ) AS b"
+      ." ( SELECT student,page,copy,image FROM $t_zone WHERE type=? ) AS b"
       ." ON a.student=b.student AND a.page=b.page AND a.copy=b.copy"
      },
      'photocopy'=>{'sql'=>"SELECT COUNT(*) FROM $t_page WHERE copy>0"},
+     'zonesBBox'=>{'sql'=>"SELECT z.id_a AS question,z.id_b AS answer,"
+		   ."   min(p.x) AS xmin,max(p.x) AS xmax,"
+		   ."   min(p.y) AS ymin,max(p.y) AS ymax"
+		   ." FROM $t_zone AS z,$t_position as p"
+		   ." ON z.zoneid=p.zoneid"
+		   ." WHERE z.student=? AND z.page=? AND z.copy=?"
+		   ."   AND z.type=? AND p.type=?"
+		   ." GROUP BY z.zoneid"},
+     'zonesCorners'=>{'sql'=>"SELECT z.id_a AS question,z.id_b AS answer,"
+		      ."   p.x AS x,p.y AS y,p.corner AS corner"
+		      ." FROM $t_zone AS z,$t_position as p"
+		      ." ON z.zoneid=p.zoneid"
+		      ." WHERE z.student=? AND z.page=? AND z.copy=?"
+		      ."   AND z.type=? AND p.type=?"
+		      ." ORDER BY z.zoneid,p.corner"},
     };
   $self->{'statements'}->{'pageSummary'}=
     {'sql'=>$self->{'statements'}->{'pagesSummary'}->{'sql'}
      ." WHERE student=? AND page=? AND copy=?"};
+  $self->{'statements'}->{'pagesSummary'}->{'sql'}.=" ORDER BY student,page,copy";
 }
 
 # get_page($student,$page,$copy) returns all columns from the row with
@@ -658,13 +727,34 @@ sub set_zone_manual {
   $self->statement('setZoneManual')->execute($manual,$zoneid);
 }
 
-# set_zone_auto(...,$total,$black,$image) sets automated data capture
-# results for a particular zone.
+# set_zone_auto(...,$total,$black,$image,$image_data) sets automated
+# data capture results for a particular zone.
 
 sub set_zone_auto {
-  my ($self,$student,$page,$copy,$type,$id_a,$id_b,$total,$black,$image)=@_;
-  $self->statement('setZoneAuto')->execute($total,$black,$image,
-					   $self->get_zoneid($student,$page,$copy,$type,$id_a,$id_b,1));
+  my ($self,$student,$page,$copy,$type,$id_a,$id_b,$total,$black,$image,$image_data)=@_;
+  $self->set_zone_auto_id($self->get_zoneid($student,$page,$copy,$type,$id_a,$id_b,1),
+			  $total,$black,$image,$image_data);
+}
+
+sub set_zone_auto_id {
+  my ($self,$zoneid,$total,$black,$image,$image_data)=@_;
+  my $sth=$self->statement('setZoneAuto');
+  $sth->bind_param(1,$total);
+  $sth->bind_param(2,$black);
+  $sth->bind_param(3,$image);
+  $sth->bind_param(4,$image_data,SQL_BLOB);
+  $sth->bind_param(5,$zoneid);
+  $sth->execute();
+}
+
+sub set_zone_auto_id_without_imagedata {
+  my ($self,$zoneid,$total,$black,$image)=@_;
+  my $sth=$self->statement('setZoneAutoPrim');
+  $sth->bind_param(1,$total);
+  $sth->bind_param(2,$black);
+  $sth->bind_param(3,$image);
+  $sth->bind_param(4,$zoneid);
+  $sth->execute();
 }
 
 # n_pages returns the number of copies for which a data capture (manual
@@ -857,6 +947,15 @@ sub ticked_sums {
 	 );
 }
 
+# has_answer_zero($student,$copy,$question) returns true if the
+# requested question has an answer "none of the above are correct".
+
+sub has_answer_zero {
+  my ($self,$student,$copy,$question)=@_;
+  return($self->sql_single($self->statement('questionHasZero'),
+			   $student,$copy,ZONE_BOX,$question));
+}
+
 # ticked_list($student,$copy,$question,$darkness_threshold) returns a
 # list with the ticked results for all the answers boxes from a
 # particular question. Answers are ordered with the answer number, so
@@ -876,8 +975,7 @@ sub ticked_list {
 sub ticked_list_0 {
   my ($self,$student,$copy,$question,$darkness_threshold)=@_;
   my @tl=$self->ticked_list($student,$copy,$question,$darkness_threshold);
-  if($self->sql_single($self->statement('questionHasZero'),
-		       $student,$copy,ZONE_BOX,$question)) {
+  if($self->has_answer_zero($student,$copy,$question)) {
     my $zero=shift @tl;
     push @tl,$zero;
   }
@@ -1187,6 +1285,48 @@ sub get_namefields {
 					{ Slice => {} },ZONE_NAME));
 }
 
+# get_zones_bbox($student,$page,$copy,$zone_type) returns an arrayref
+# giving, for each zone with type $zone_type (defaulting to ZONE_BOX)
+# on scanned page ($student,$page,$copy), the bounding box on the scan.
+#
+# For example:
+#
+# [{question=>1,answer=>1,
+#   xmin=>1646.92,xmax=>1684.57,ymin=>1608.12,ymax=>1645.38},
+#  {question=>1,answer=>2,
+#   xmin=>1591.68,xmax=>1629.33,ymin=>1517.17,ymax=>1554.43},
+# ]
+
+sub get_zones_bbox {
+  my ($self,$student,$page,$copy,$zone_type)=@_;
+  $zone_type=ZONE_BOX if(!defined($zone_type));
+  return($self->dbh->selectall_arrayref($self->statement('zonesBBox'),
+					{ Slice => {} },
+					$student,$page,$copy,
+					$zone_type,POSITION_BOX));
+}
+
+# get_zones_corners($student,$page,$copy,$zone_type) returns an
+# arrayref giving, for each zone with type $zone_type (defaulting to
+# ZONE_BOX) on scanned page ($student,$page,$copy), the corners on the
+# scan. In the resulting array, corners are ordered.
+#
+# For example:
+#
+# [{zoneid=>156,question=>1,answer=>1,x=>1646.92,y=>1608.12,corner=>1},
+#  {zoneid=>156,question=>1,answer=>1,x=>1667.12,y=>1610.43,corner=>2},
+#  ...
+# ]
+
+sub get_zones_corners {
+  my ($self,$student,$page,$copy,$zone_type)=@_;
+  $zone_type=ZONE_BOX if(!defined($zone_type));
+  return($self->dbh->selectall_arrayref($self->statement('zonesCorners'),
+					{ Slice => {} },
+					$student,$page,$copy,
+					$zone_type,POSITION_BOX));
+}
+
 # n_photocopy() returns the number of captures made on a photocopy of
 # the subject, that is made with mode 'Some sheets were photocopied'
 # (these captures are identified by a positive 'copy' value).
@@ -1196,7 +1336,7 @@ sub n_photocopy {
   return($self->sql_single($self->statement('photocopy')));
 }
 
-# clear_all clears all the layout data tables.
+# clear_all() clears all the layout data tables.
 
 sub clear_all {
     my ($self)=@_;
@@ -1205,4 +1345,51 @@ sub clear_all {
     }
 }
 
+# get_image_paths($type) returns an arrayref with image paths of all
+# zones of type $type, like
+#
+# [{zoneid=>153,image=>"zone-image-001.png"},
+#  ...
+# ]
+
+sub get_image_paths {
+  my ($self,$type)=@_;
+  return($self->dbh->selectall_arrayref($self->statement('imagePaths'),
+					{ Slice => {} },$type));
+}
+
+# set_image($zoneid,$imagedata) sets the imagedata (the image content
+# as a blob) for zone $zoneid.
+
+sub set_image {
+  my ($self,$zoneid,$imagedata)=@_;
+  my $sth=$self->statement('setImage');
+  $sth->bind_param(1,$imagedata,SQL_BLOB);
+  $sth->bind_param(2,$zoneid);
+  $sth->execute();
+}
+
+# zooms_total_size_transaction() returns the sum of all zooms
+# images stored in the database, in bytes.
+
+sub zooms_total_size_transaction {
+  my ($self)=@_;
+  $self->begin_read_transaction('ztsz');
+  my $s=$self->sql_single($self->statement('zoomsTotalSize'),ZONE_BOX);
+  $self->end_transaction('ztsz');
+  return($s);
+}
+
+# zooms_cleanup_transaction() deletes all zooms images from the
+# database.
+
+sub zooms_cleanup_transaction {
+  my ($self)=@_;
+  $self->begin_transaction('zcln');
+  my $n=$self->statement('zoomsCleanup')->execute(ZONE_BOX);
+  $self->end_transaction('zcln');
+  $self->vacuum();
+  return($n);
+}
+
 1;
