@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -33,6 +33,9 @@ use_gettext;
 sub new {
     my $class = shift;
     my $self  = $class->SUPER::new();
+
+    # list of available global options:
+
     $self->{'options_names'}=[qw/Title Presentation Code Lang Font
 				 BoxColor PaperSize
 				 AnswerSheetTitle AnswerSheetPresentation
@@ -44,15 +47,29 @@ sub new {
 				 LaTeXEngine xltxtra
 				 ShuffleQuestions Columns QuestionBlocks
 				 Arabic ArabicFont
+				 Disable
+				 ManualDuplex SingleSided
+				 L-OpenText L-OpenReserved
 				/];
+
+    # from these options, which ones are boolean valued?
+
     $self->{'options_boolean'}=[qw/LaTeX xltxtra
 				   ShuffleQuestions QuestionBlocks
 				   CompleteMulti SeparateAnswerSheet
 				   Arabic
+				   ManualDuplex SingleSided
 				  /];
+
+    # current groups list
     $self->{'groups'}=[];
+    # maximum of digits that a hrizontal code can handle
     $self->{'maxhorizcode'}=6;
+    # options values
     $self->{'options'}={};
+
+    # default values for options:
+
     $self->{'default_options'}=
       {'latexengine'=>'xelatex','xltxtra'=>1,
        'questionblocks'=>1,'shufflequestions'=>1,
@@ -62,21 +79,38 @@ sub new {
        'defaultscoringm'=>'haut=2',
        'l-name'=>__("Name and surname"),
        'l-student'=>__("Please code your student number opposite, and write your name in the box below."),
+       'disable'=>'',
+       'manualduplex'=>'',
+       'singlesided'=>'',
       };
+
+    # List of modules to be used when parsing (see parse_*
+    # corresponding methods for implementation). Modules in the
+    # Disable global option won't be used.
+
+    $self->{'parse_modules'}=['local_latex','images','embf','text'];
+
+    # current question number among questions for which no ID is given
+    # in the source
     $self->{'qid'}=0;
+
     bless ($self, $class);
     return $self;
 }
 
+# arabic default localisation texts
 my %l_arabic=('l-question'=>'السؤال',
 	      'l-none'=>'لا توجد اجابة صحيحة',
 	      );
 
+# arabic alphabet to replace letters ABCDEF... in the boxes
 my @alphabet_arabic=('أ','ب','ج','د','ه','و','ز','ح','ط','ي','ك','ل',
 		     'م','ن','س','ع','ف','ص','ق','ر','ش','ت','ث','خ',
 		     'ذ','ض','ظ','غ',
 		     );
 
+# parse boolean options to get 0 (from empty value, "NO" or " FALSE")
+# or 1 (from other values).
 sub parse_bool {
   my ($b)=@_;
   if($b =~ /^\s*(no|false|0)\s*$/i) {
@@ -86,56 +120,74 @@ sub parse_bool {
   }
 }
 
+# transforms the list of current options values to make it coherent
 sub parse_options {
   my ($self)=@_;
+
+  # boolean options are parsed to get 0 or 1
   for my $n (@{$self->{'options_boolean'}}) {
     $self->{'options'}->{lc($n)}=parse_bool($self->{'options'}->{lc($n)});
   }
 
+  # if AR language is selected, apply default arabic localisation
+  # options and set 'arabic' option to 1
   if($self->{'options'}->{'lang'} eq 'AR') {
     for (keys %l_arabic) {
       $self->{'options'}->{$_}=$l_arabic{$_}
 	if(!$self->{'options'}->{$_});
     }
+    $self->{'options'}->{'arabic'}=1;
   }
 
+  # if JA language is selected, switch to 'platex+dvipdfmx' LaTeX
+  # engine, remove default font and don't use xltxtra LaTeX package
   if($self->{'options'}->{'lang'} eq 'JA') {
     $self->{'default_options'}->{'latexengine'}='platex+dvipdfmx';
     $self->{'default_options'}->{'font'}='';
     $self->{'default_options'}->{'xltxtra'}='';
   }
 
+  # set options values to default if not defined in the source
   for my $k (keys %{$self->{'default_options'}}) {
     $self->{'options'}->{$k}=$self->{'default_options'}->{$k}
       if(!defined($self->{'options'}->{$k}));
   }
 
+  # relay LaTeX engine option to the project itself
   $self->set_project_option('moteur_latex_b',
 			    $self->{'options'}->{'latexengine'});
 }
 
+# adds an object (hash) to a container (list) and returns the
+# corresponding new hashref
 sub add_object {
   my ($container,%object)=@_;
   push @$container,{%object};
   return($container->[$#$container]);
 }
 
+# adds a group to current state
 sub add_group {
   my ($self,%g)=@_;
   add_object($self->{'groups'},%g);
 }
 
+# cleanup for text: removes leading and trailing spaces
 sub value_cleanup {
   my ($self,$v)=@_;
   $$v =~ s/^\s+//;
   $$v =~ s/\s+$//;
 }
 
+# issue an AMC-TXT (syntax) error to be shown by the GUI
 sub parse_error {
   my ($self,$text)=@_;
   $self->error("<i>AMC-TXT(".sprintf(__('Line %d'),$.).")</i> ".$text);
 }
 
+# check that a set of answers for a given question is coherent: one
+# need more than one answer, and a simple question needs one and only
+# one correct answer.
 sub check_answers {
   my ($self,$question)=@_;
   if($question) {
@@ -157,21 +209,24 @@ sub check_answers {
   }
 }
 
+# parse the whole source file to a data tree ($self->{'groups'})
 sub read_source {
   my ($self,$input_file)=@_;
 
   my %opts=();
-  my $follow='';
-  my $group='';
-  my $question='';
+  my $follow=''; # variable where to add content comming from
+                 # following lines
+  my $group='';  # current group
+  my $question=''; # current question
 
+  # regexp that matches an option name:
   my $opt_re='('.join('|',@{$self->{'options_names'}}).')';
 
   open(IN,"<:utf8",$input_file);
  LINE: while(<IN>) {
     chomp;
 
-    # comments
+    # removes comments
     s/^\s*\#.*//;
 
     # groups
@@ -204,12 +259,13 @@ sub read_source {
     }
 
     # questions
-    if(/^\s*(\*{1,2})(?:\[([^]]*)\])?(?:\{([^\}]*)\})?\s*(.*)/) {
+    if(/^\s*(\*{1,2})(?:<([^>]*)>)?(?:\[([^]]*)\])?(?:\{([^\}]*)\})?\s*(.*)/) {
       $self->check_answers($question);
       my $star=$1;
-      my $text=$4;
-      my $scoring=$3;
-      my @opts=split(/,+/,$2);
+      my $angles=$2;
+      my @opts=split(/,+/,$3);
+      my $scoring=$4;
+      my $text=$5;
       my %oo=();
       for (@opts) {
 	if(/^([^=]+)=(.*)/) {
@@ -223,6 +279,7 @@ sub read_source {
       }
       $question=add_object($group->{'questions'},
 			   'multiple'=>length($star)==2,
+			   'open'=>$angles,
 			   'scoring'=>$scoring,
 			   'text'=>$text,'answers'=>[],%oo);
       $self->value_cleanup($follow);
@@ -231,11 +288,16 @@ sub read_source {
     }
 
     # answers
-    if(/^\s*(\+|-)(?:\{([^\}]*)\})?\s*(.*)/) {
+    if(/^\s*(\+|-)(?:\[([^]]*)\])?(?:\{([^\}]*)\})?\s*(.*)/) {
       if($question) {
+	my $sign=$1;
+	my $letter=$2;
+	my $scoring=$3;
+	my $text=$4;
 	my $a=add_object($question->{'answers'},
-			 'text'=>$3,'correct'=>($1 eq '+'),
-			 'scoring'=>$2);
+			 'text'=>$text,'correct'=>($sign eq '+'),
+			 'letter'=>$letter,
+			 'scoring'=>$scoring);
 	$self->value_cleanup($follow);
 	$follow=\$a->{'text'};
       } else {
@@ -255,29 +317,184 @@ sub read_source {
   close(IN);
 }
 
+# bold font LaTeX command, not used in arabic language because there
+# is often no bold font arabic font.
 sub bf_or {
   my($self,$replace,$bf)=@_;
   return($self->{'options'}->{'arabic'}
 	 ? $replace : ($bf ? $bf : "\\bf"));
 }
 
+####################################################################
+# PARSING METHODS
+#
+# ARGUMENT: list of components
+# RETURN VALUE: list of parsed components
+#
+# A component is a {type=>TYPE,string=>STRING} hashref, where
+# * TYPE is the component type: 'txt' for AMC-TXT text, 'latex' for
+#   LaTeX code.
+# * STRING is the component string
+#
+# When all parsing methods has been applied, the result must be a list
+# of 'latex' components, that will be concatenated to get the LaTeX
+# code corresponding to the source file.
+
+# parse_images extracts an image specification (like '!image.jpg!')
+# from 'txt' components, and replaces the component with 3 components:
+# the preceding text, the LaTeX code that inserts the image, and the
+# following text
+sub parse_images {
+  my ($self,@components)=@_;
+  my @o=();
+  for my $c (@components) {
+    if($c->{'type'} eq 'txt') {
+      my $s=$c->{'string'};
+      while($s =~ /!(?:\{([^!\s]+)\})?(?:\[([^!\s]+)\])?([^!\s]+)!/p) {
+	my $options=$1;
+	my $ig_options=$2;
+	my $path=$3;
+	my $before=${^PREMATCH};
+	my $after=${^POSTMATCH};
+	push @o,{'type'=>'txt','string'=>$before};
+	my $l="\\includegraphics".($ig_options ? '['.$ig_options.']' : '')
+	  .'{'.$path.'}';
+	if($options =~ /\bcenter\b/) {
+	  $l="\\begin{center}$l\\end{center}";
+	}
+	push @o,{'type'=>'latex','string'=>$l};
+	$s=$after;
+      }
+      push @o,{'type'=>'txt','string'=>$s};
+    } else {
+      push @o,$c;
+    }
+  }
+  return(@o);
+}
+
+# parse_local_latex extracts a local LaTeX specification (like
+# '[[$f(x)$]]') from 'txt' components
+sub parse_local_latex {
+  my ($self,@components)=@_;
+  my @o=();
+  for my $c (@components) {
+    if($c->{'type'} eq 'txt') {
+      my $s=$c->{'string'};
+      while($s =~ /\[\[(((?!\]\]).)+)\]\]/p) {
+	my $latex=$1;
+	my $before=${^PREMATCH};
+	my $after=${^POSTMATCH};
+	push @o,{'type'=>'txt','string'=>$before};
+	push @o,{'type'=>'latex','string'=>$latex};
+	$s=$after;
+      }
+      push @o,{'type'=>'txt','string'=>$s};
+    } else {
+      push @o,$c;
+    }
+  }
+  return(@o);
+}
+
+# parse_embf inserts LaTeX commands to switch to italic or bold font
+# when '[_ ... _]' or '[* ... *]' constructs are used in AMC-TXT.
+sub parse_embf {
+  my ($self,@components)=@_;
+  my @o=();
+  for my $c (@components) {
+    if($c->{'type'} eq 'txt') {
+      my $s=$c->{'string'};
+      while($s =~ /\[([_\*])(((?!\]\]).)+?)\1\]/p) {
+	my $modif=$1;
+	my $modif_latex;
+	my $text=$2;
+	my $before=${^PREMATCH};
+	my $after=${^POSTMATCH};
+	if($modif eq '_') {
+	  $modif_latex='it';
+	} else {
+	  $modif_latex='bf';
+	}
+	push @o,{'type'=>'txt','string'=>$before};
+	push @o,{'type'=>'latex','string'=>"\\text".$modif_latex."{"};
+	push @o,$self->parse_embf({'type'=>'txt','string'=>$text});
+	push @o,{'type'=>'latex','string'=>"}"};
+	$s=$after;
+      }
+      push @o,{'type'=>'txt','string'=>$s};
+    } else {
+      push @o,$c;
+    }
+  }
+  return(@o);
+}
+
+# parse_text transforms plain text (with no special constructs) to
+# LaTeX code, escaping some characters that have special meaning for
+# LaTeX.
+sub parse_text {
+  my ($self,@components)=@_;
+  my @o=();
+  for my $c (@components) {
+    if($c->{'type'} eq 'txt') {
+      my $s=$c->{'string'};
+      if(! $self->{'options'}->{'latex'}) {
+	$s =~ s/\\/\\(\\backslash\\)/g;
+	$s =~ s/~/\\(\\sim\\)/g;
+	$s =~ s/\*/\\(\\ast\\)/g;
+	$s =~ s/([&{}\#_%])/\\\1/g;
+	$s =~ s/-/-{}/g;
+	$s =~ s/\$/\\textdollar{}/g;
+	$s =~ s/\^/\\textasciicircum{}/g;
+      }
+      push @o,{'type'=>'latex','string'=>$s};
+    } else {
+      push @o,$c;
+    }
+  }
+  return(@o);
+}
+
+# parse_all calls all requested parse_* methods in turn (when they are
+# not disabled by the Disable global option).
+sub parse_all {
+  my ($self,@components)=@_;
+  for my $m (@{$self->{'parse_modules'}}) {
+    my $mm='parse_'.$m;
+    @components=$self->$mm(@components)
+      if($self->{'options'}->{'disable'} !~ /\b$m\b/);
+  }
+  return(@components);
+}
+
+# Checks that the resulting components list has only 'latex' components
+sub check_latex {
+  my ($self,@components)=@_;
+  my @s=();
+  for my $c (@components) {
+    if($c->{'type'} ne 'latex') {
+      debug_and_stderr "ERR(FILTER): non-latex resulting component (".
+	$c->{'type'}."): ".$c->{'string'};
+    }
+    push @s,$c->{'string'};
+  }
+  return(@s);
+}
+
+# converts AMC-TXT string to a LaTeX string: make a single 'txt'
+# component, apply parse_all and check_latex, and then concatenates
+# the components
 sub format_text {
   my ($self,$t)=@_;
   $t =~ s/^\s+//;
   $t =~ s/\s+$//;
-  if($self->{'options'}->{'latex'}) {
-  } else {
-    $t =~ s/\\/\\(\\backslash\\)/g;
-    $t =~ s/~/\\(\\sim\\)/g;
-    $t =~ s/\*/\\(\\ast\\)/g;
-    $t =~ s/([&{}\#_%])/\\\1/g;
-    $t =~ s/-/-{}/g;
-    $t =~ s/\$/\\textdollar{}/g;
-    $t =~ s/\^/\\textasciicircum{}/g;
-  }
-  return($t);
+
+  return(join('',$self->check_latex($self->parse_all({'type'=>'txt','string'=>$t}))));
 }
 
+# builds the LaTeX scoring command from the question's scoring
+# strategy (or the default scoring strategy)
 sub scoring_string {
   my ($self,$obj,$type)=@_;
   my $s=$obj->{'scoring'}
@@ -285,15 +502,20 @@ sub scoring_string {
   return($s ? "\\scoring{$s}" : "");
 }
 
+# builds the LaTeX code for an answer: \correctchoice or \wrongchoice,
+# followed by the scoring command
 sub format_answer {
   my ($self,$a)=@_;
-  my $t='\\'.($a->{'correct'} ? 'correct' : 'wrong').'choice{'
+  my $t='\\'.($a->{'correct'} ? 'correct' : 'wrong').'choice';
+  $t.='['.$a->{letter}.']' if($a->{letter} ne '');
+  $t.='{'
     .$self->format_text($a->{'text'})."}";
   $t.=$self->scoring_string($a,'a');
   $t.="\n";
   return($t);
 }
 
+# builds the LaTeX code for the question (including answers)
 sub format_question {
   my ($self,$q)=@_;
   my $qid=$q->{'id'};
@@ -308,15 +530,23 @@ sub format_question {
   $t.=$self->scoring_string($q,($q->{'multiple'} ? 'm' : 's'));
   $t.="\n";
   $t.=$self->format_text($q->{'text'})."\n";
-  $t.="\\begin{multicols}{".$q->{'columns'}."}\n"
-    if($q->{'columns'}>1);
-  $t.="\\begin{choices$ct}".($q->{'ordered'} ? "[o]" : "")."\n";
+  if($q->{open} ne '') {
+    $t.="\\AMCOpen{".$q->{open}."}{";
+  } else {
+    $t.="\\begin{multicols}{".$q->{'columns'}."}\n"
+      if($q->{'columns'}>1);
+    $t.="\\begin{choices$ct}".($q->{'ordered'} ? "[o]" : "")."\n";
+  }
   for my $a (@{$q->{'answers'}}) {
     $t.=$self->format_answer($a);
   }
-  $t.="\\end{choices$ct}\n";
-  $t.="\\end{multicols}\n"
-    if($q->{'columns'}>1);
+  if($q->{open} ne '') {
+    $t.="}\n";
+  } else {
+    $t.="\\end{choices$ct}\n";
+    $t.="\\end{multicols}\n"
+      if($q->{'columns'}>1);
+  }
   $t.="\\end{question".$mult."}";
   $t.="\\end{arab}"
     if($self->{'options'}->{'arabic'} && $self->bidi_year()<2011);
@@ -324,6 +554,8 @@ sub format_question {
   return($t);
 }
 
+# returns the group name, defaulting to 'groupX' (where X is a letter
+# beginnig at A)
 sub group_name {
   my ($self,$group)=@_;
   if(!$group->{'name'}) {
@@ -332,6 +564,8 @@ sub group_name {
   return($group->{'name'});
 }
 
+# gets the Year for the version of installed bidi.sty. This will be
+# used to adapt the LaTeX code to version before or after 2011
 sub bidi_year {
   my ($self)=@_;
   if(!$self->{'bidiyear'}) {
@@ -350,6 +584,7 @@ sub bidi_year {
   return($self->{'bidiyear'});
 }
 
+# builds and returns the LaTeX header
 sub file_header {
   my ($self)=@_;
   my $t='';
@@ -408,6 +643,10 @@ sub file_header {
   $t .= "\\AMCtext{none}{"
     .$self->format_text($self->{'options'}->{'l-none'})."}\n"
     if($self->{'options'}->{'l-none'});
+  $t .= "\\def\\AMCotextGoto{\\par ".$self->format_text($self->{'options'}->{'l-opentext'})."}\n"
+    if($self->{'options'}->{'l-opentext'});
+  $t .= "\\def\\AMCotextReserved{".$self->format_text($self->{'options'}->{'l-openreserved'})."}\n"
+    if($self->{'options'}->{'l-openreserved'});
 
   $t.="\\def\\AMCbeginQuestion#1#2{\\par\\noindent{"
     .$self->bf_or("\\Large")." "
@@ -429,6 +668,8 @@ sub file_header {
   return($t);
 }
 
+# Builds and returns the question sheet head (or the answer sheet head
+# if $answersheet is true)
 sub page_header {
   my ($self,$answersheet)=@_;
   my $t="";
@@ -457,6 +698,7 @@ sub page_header {
   return($t);
 }
 
+# Builds and returns the name field LaTeX code
 sub full_namefield {
   my ($self,$n_ligs)=@_;
   my $t='';
@@ -470,6 +712,8 @@ sub full_namefield {
   return($t);
 }
 
+# Builds and returns the student identification block LaTeX code (with
+# name field and AMCcode if requested)
 sub student_block {
   my ($self)=@_;
   my $t='';
@@ -513,6 +757,7 @@ sub student_block {
   return($t);
 }
 
+# writes the LaTeX output file
 sub write_latex {
   my ($self,$output_file)=@_;
 
@@ -575,7 +820,11 @@ sub write_latex {
   }
 
   if($self->{'options'}->{'separateanswersheet'}) {
-    print OUT "\\AMCcleardoublepage\n";
+    if($self->{'options'}->{'singlesided'}) {
+      print OUT "\n\\clearpage\n\n";
+    } else {
+      print OUT "\n\\AMCcleardoublepage\n\n";
+    }
     print OUT "\\AMCformBegin\n";
 
     print OUT "\\begin{arab}" if($self->{'options'}->{'arabic'});
@@ -597,11 +846,15 @@ sub write_latex {
     print OUT "\\end{arab}" if($self->{'options'}->{'arabic'});
   }
 
+  print OUT "\\AMCcleardoublepage\n" if($self->{'options'}->{'manualduplex'});
+
   print OUT "}\n";
   print OUT "\\end{document}\n";
   close(OUT);
 }
 
+# Checks that the requested prerequisites (in fact, fonts) are
+# installed on the system
 sub check {
   my ($self)=@_;
   my @cf=('font');
@@ -618,6 +871,7 @@ sub check {
   $self->error(sprintf(__("The following fonts does not seem to be installed on the system: <b>%s</b>."),join(', ',@mf))) if(@mf);
 }
 
+# Whole filter processing
 sub filter {
   my ($self,$input_file,$output_file)=@_;
   $self->read_source($input_file);
