@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2012-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -20,6 +20,8 @@
 
 package AMC::Gui::Zooms;
 
+use File::Spec::Functions qw/tmpdir/;
+
 use AMC::Basic;
 use AMC::DataModule::capture ':zone';
 
@@ -48,6 +50,7 @@ sub new {
 	'data-dir'=>'',
 	'size-prefs'=>'',
 	'encodage_interne'=>'UTF-8',
+	'list_view'=>'',
     };
 
     for (keys %o) {
@@ -90,7 +93,7 @@ sub new {
     $self->{'gui'}->set_translation_domain('auto-multiple-choice');
     $self->{'gui'}->add_from_file($glade_xml);
 
-    for(qw/main_window zooms_table_0 zooms_table_1 decoupage view_0 view_1 scrolled_0 scrolled_1 label_0 label_1 event_0 event_1 button_apply button_close info/) {
+    for(qw/main_window zooms_table_0 zooms_table_1 decoupage view_0 view_1 scrolled_0 scrolled_1 label_0 label_1 event_0 event_1 button_apply button_close info button_previous button_next/) {
 	$self->{$_}=$self->{'gui'}->get_object($_);
     }
 
@@ -152,38 +155,61 @@ sub scrolled_update {
   }
 }
 
+sub get_page_info {
+  my ($self)=@_;
+
+  $self->{'page_info'}
+    =$self->{'_capture'}->get_page(@{$self->{'page_id'}});
+}
+
+sub affect_box {
+  my ($self,$z)=@_;
+
+  my $id=$z->{'id_a'}.'-'.$z->{'id_b'};
+  my $auto_pos=($z->{'black'} > $z->{'total'}*$self->{'seuil'} ? 1 : 0);
+  my $eff_pos=($self->{'page_info'}->{'timestamp_manual'}
+	       && $z->{'manual'}>=0 ? $z->{'manual'} :
+	       $auto_pos);
+  $self->{'eff_pos'}->{$id}=$eff_pos;
+  $self->{'auto_pos'}->{$id}=$auto_pos;
+}
+
 sub load_positions {
   my ($self)=@_;
   $self->{'_capture'}->begin_read_transaction;
 
-  my $page=$self->{'_capture'}->get_page(@{$self->{'page_id'}});
+  $self->get_page_info();
 
   my $sth=$self->{'_capture'}->statement('pageZonesD');
   $sth->execute(@{$self->{'page_id'}},ZONE_BOX);
   while (my $z = $sth->fetchrow_hashref) {
-    my $id=$z->{'id_a'}.'-'.$z->{'id_b'};
-    my $auto_pos=($z->{'black'} > $z->{'total'}*$self->{'seuil'} ? 1 : 0);
-    my $eff_pos=($page->{'timestamp_manual'} && $z->{'manual'}>=0 ? $z->{'manual'} :
-		 $auto_pos);
-    $self->{'eff_pos'}->{$id}=$eff_pos;
-    $self->{'auto_pos'}->{$id}=$auto_pos;
+    $self->affect_box($z);
   }
 
   $self->{'_capture'}->end_transaction;
 }
 
 sub safe_pixbuf {
-  my ($self,$file)=@_;
+  my ($self,$image)=@_;
   my $p='';
-  if(-f $file) {
-    # Try to load PNG file. This can fail in case of problem (for
-    # example if mimetype was not detected correctly due to special
-    # file content matching other mime types).
-    eval { $p=Gtk2::Gdk::Pixbuf->new_from_file($file); };
+  if($image) {
+    # first try with a PixbufLoader
+
+    my $pxl=Gtk2::Gdk::PixbufLoader->new;
+    $pxl->write($image);
+    $pxl->close();
+    $p=$pxl->get_pixbuf();
     return($p,1) if($p);
-    # Try using Graphics::Magick to convert PNG->XPM
-    my $i=magick_perl_module()->new;
-    $i->Read($file);
+
+    # Then try using Graphics::Magick to convert to XPM
+    my $i=magick_perl_module()->new();
+    $i->BlobToImage($image);
+    if( ! $i[0]) {
+      # Try using temporary file to do the same
+      my $tf=tmpdir()."/AMC-tempzoom";
+      open TZ,">$tf";binmode TZ;print TZ $image;close TZ;
+      $i->Read($tf);
+    }
     my @b=$i->ImageToBlob("magick"=>'xpm');
     if($b[0]) {
       $b[0] =~ s:/\*.*\*/::g;
@@ -215,21 +241,22 @@ sub load_boxes {
 
     my @ids;
 
-    $self->load_positions;
-
     $self->{'_capture'}->begin_read_transaction;
 
-    my $sth=$self->{'_capture'}->statement('pageZonesD');
+    $self->get_page_info();
+
+    my $sth=$self->{'_capture'}->statement('pageZonesDI');
     $sth->execute(@{$self->{'page_id'}},ZONE_BOX);
     while (my $z = $sth->fetchrow_hashref) {
 
-      my $id=$z->{'id_a'}.'-'.$z->{'id_b'};
-      my $fid=$self->{'zooms_dir'}."/".$z->{'image'};
+      $self->affect_box($z);
 
-      if(-f $fid) {
+      my $id=$z->{'id_a'}.'-'.$z->{'id_b'};
+
+      if($z->{imagedata}) {
 
 	($self->{'pb_src'}->{$id},$self->{'real_src'}->{$id})
-	  =$self->safe_pixbuf($fid);
+	  =$self->safe_pixbuf($z->{imagedata});
 
 	$self->{'image'}->{$id}=Gtk2::Image->new();
 
@@ -266,7 +293,7 @@ sub load_boxes {
 
 	push @ids,$id;
       } else {
-	debug_and_stderr "Zoom file not found: $fid";
+	debug_and_stderr "No zoom image: $id";
       }
     }
 
@@ -323,7 +350,7 @@ sub page {
 	    ->new_with_markup($self->{'main_window'},
 			      'destroy-with-parent',
 			      'warning','yes-no',
-			      __("You moved some boxes to correct automatic data query, but this work is not saved yet.")." ".__("Dou you want to save these modifications before looking at another page?")
+			      __("You moved some boxes to correct automatic data query, but this work is not saved yet.")." ".__("Do you want to save these modifications before looking at another page?")
 	    );
 	my $reponse=$dialog->run;
 	$dialog->destroy;
@@ -465,6 +492,29 @@ sub zoom_arriere {
     $self->zoom_it();
 }
 
+sub zoom_list_previous {
+  my ($self)=@_;
+  my ($path)=$self->{'list_view'}->get_cursor();
+  if($path) {
+    if($path->prev) {
+      $self->{'list_view'}->set_cursor($path);
+    }
+  }
+}
+
+sub zoom_list_next {
+  my ($self)=@_;
+  my ($path)=$self->{'list_view'}->get_cursor();
+  if($path) {
+    my $path_next=Gtk2::TreePath->new ($path->to_string);
+    $path_next->next();
+    $self->{'list_view'}->set_cursor($path_next);
+    ($path_next)=$self->{'list_view'}->get_cursor();
+    $self->{'list_view'}->set_cursor($path)
+      if(!$path_next);
+  }
+}
+
 sub quit {
     my ($self)=@_;
 
