@@ -1,6 +1,6 @@
 #! /usr/bin/perl
 #
-# Copyright (C) 2010-2012 Alexis Bienvenue <paamc@passoire.fr>
+# Copyright (C) 2010-2013 Alexis Bienvenue <paamc@passoire.fr>
 #
 # This file is part of Auto-Multiple-Choice
 #
@@ -21,10 +21,15 @@
 use Locale::PO;
 use Encode;
 use utf8;
+use Getopt::Long;
+
+my $comment_only='';
+
+GetOptions("comment-only!"=>\$comment_only);
 
 my ($from,$to)=@ARGV;
 
-$v="1.0.1065 (svn: 1065)";
+$v="1.1.1359 (svn: 1359)";
 
 chomp($date=`date "+%Y-%m-%d %H:%M %z"`);
 
@@ -32,16 +37,19 @@ my $aref = Locale::PO->load_file_asarray($from);
 
 my @ok=grep { $_->msgid() !~ /^\"?gtk-/ } @$aref;
 
-$ok[0]->comment(encode_utf8("Auto Multiple Choice\nCopyright 2008-2012 Alexis Bienvenüe\nThis file is distributed under the same license as the AMC software"));
-$ok[0]->msgstr("Project-Id-Version: $v\\n" .
-	       "Report-Msgid-Bugs-To: paamc\@passoire.fr\\n" .
-	       "POT-Creation-Date: $date\\n" .
-	       "PO-Revision-Date: YEAR-MO-DA HO:MI +ZONE\\n" .
-	       "Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n" .
-	       "Language-Team: LANGUAGE <LL@li.org>\\n" .
-	       "MIME-Version: 1.0\\n" .
-	       "Content-Type: text/plain; charset=UTF-8\\n" .
-	       "Content-Transfer-Encoding: 8bit\\n");
+$ok[0]->comment(encode_utf8("Auto Multiple Choice\nCopyright 2008-2013 Alexis Bienvenüe\nThis file is distributed under the same license as the AMC software"));
+
+if(!$comment_only) {
+  $ok[0]->msgstr("Project-Id-Version: $v\\n" .
+		 "Report-Msgid-Bugs-To: paamc\@passoire.fr\\n" .
+		 "POT-Creation-Date: $date\\n" .
+		 "PO-Revision-Date: YEAR-MO-DA HO:MI +ZONE\\n" .
+		 "Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n" .
+		 "Language-Team: LANGUAGE <LL@li.org>\\n" .
+		 "MIME-Version: 1.0\\n" .
+		 "Content-Type: text/plain; charset=UTF-8\\n" .
+		 "Content-Transfer-Encoding: 8bit\\n");
+}
 
 Locale::PO->save_file_fromarray($to,\@ok);
 
